import unittest

from acdh_wikidata_pyutils import NoWikiDataUrlException, WikiDataPerson, WikiDataPlace

ARTHUR_SCHNITZLER_URL = "https://www.wikidata.org/wiki/Q44331"
ARTHUR_SCHNITZLER_ID = "Q44331"
POOR_DATA_URL = "https://www.wikidata.org/wiki/Q122733648"
LINZ_URL = "https://www.wikidata.org/wiki/Q41329"

ARTHUR_SCHNITZLER = WikiDataPerson(ARTHUR_SCHNITZLER_URL)
POOR_DATA_ITEM = WikiDataPerson(POOR_DATA_URL)
LINZ = WikiDataPlace(LINZ_URL)


class TestTestTest(unittest.TestCase):
    """Tests for `acdh-wikidata-pyutils` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_testsetup(self):
        self.assertTrue(True)

    def test_002_valid_url(self):
        item = ARTHUR_SCHNITZLER
        self.assertEqual(item.wikidata_id, ARTHUR_SCHNITZLER_ID)

    def test_003_non_valid_url(self):
        with self.assertRaises(NoWikiDataUrlException):
            WikiDataPerson(ARTHUR_SCHNITZLER_URL.replace("wikidata", "hansi"))

    def test_004_label(self):
        item = ARTHUR_SCHNITZLER
        self.assertEqual(item.label, "Arthur Schnitzler")

    def test_005_poor_data(self):
        item = POOR_DATA_ITEM
        apis_person = item.get_apis_entity()
        self.assertFalse(apis_person["first_name"])

    def test_006_no_name(self):
        item = POOR_DATA_ITEM
        apis_person = item.get_apis_entity()
        self.assertEqual(apis_person["name"], "Peter Andorfer")

    def test_007_coords(self):
        item = LINZ
        self.assertTrue("lat" in item.get_apis_entity().keys())

    def test_008_no_coords(self):
        item = WikiDataPlace("https://www.wikidata.org/wiki/Q16006181")
        self.assertFalse(item.lat)
        self.assertFalse(item.gnd_uri)
        self.assertFalse(item.geonames_uri)

    def test_008_no_ngd(self):
        item = WikiDataPerson("https://www.wikidata.org/wiki/Q16006181")
        self.assertFalse(item.gnd_uri)



from AcdhArcheAssets.uri_norm_rules import get_norm_id, get_normalized_uri
from wikidata.client import Client

WIKIDATA_URL = "https://www.wikidata.org/wiki/"
GEONAMES_URL = "https://sws.geonames.org/"
GND_URL = "https://d-nb.info/gnd/"


class NoWikiDataUrlException(Exception):
    pass


def check_url(wikidata_url):
    if "wikidata" not in wikidata_url:
        raise NoWikiDataUrlException(f"{wikidata_url} is no proper Wikidata URL")
    else:
        return get_normalized_uri(wikidata_url)


class WikiDataPlace:
    """Class to fetch and return often used data from WikiData Person entries"""

    def get_apis_entity(self):
        return {"name": self.label, "lat": self.lat, "lng": self.lng}

    def __init__(self, wikidata_url):
        self.wikidata_url = check_url(wikidata_url)
        self.wikidata_id = get_norm_id(self.wikidata_url)
        self.client = Client()
        self.entity = self.client.get(self.wikidata_id, load=True)
        self.label = str(self.entity.label)
        coordinates_prop = self.client.get("P625")
        gnd_uri_property = self.client.get("P227")
        geonames_uri_property = self.client.get("P1566")
        try:
            coordinates = self.entity[coordinates_prop]
        except KeyError:
            coordinates = False
        if coordinates:
            self.lat = coordinates.latitude
            self.lng = coordinates.longitude
        else:
            self.lat = None
            self.lng = None
        try:
            gnd_uri = self.entity[gnd_uri_property]
            self.gnd_uri = get_normalized_uri(f"{GND_URL}{gnd_uri}")
        except KeyError:
            self.gnd_uri = False
        try:
            geonames_uri = self.entity[geonames_uri_property]
            self.geonames_uri = get_normalized_uri(f"{GEONAMES_URL}{geonames_uri}")
        except KeyError:
            self.geonames_uri = False


class WikiDataPerson:
    """Class to fetch and return often used data from WikiData Person entries"""

    def get_apis_entity(self):
        return {
            "name": self.name,
            "first_name": self.first_name,
            "start_date_written": self.date_of_birth,
            "end_date_written": self.date_of_death,
            "gender": self.sex_or_gender,
        }

    def __init__(self, wikidata_url):
        self.wikidata_url = check_url(wikidata_url)
        self.wikidata_id = get_norm_id(self.wikidata_url)
        self.client = Client()
        self.entity = self.client.get(self.wikidata_id, load=True)
        self.label = str(self.entity.label)
        date_of_birth_prop = self.client.get("P569")
        date_of_death_prop = self.client.get("P570")
        place_of_birth_prop = self.client.get("P19")
        place_of_death_prop = self.client.get("P20")
        sex_or_gender_prop = self.client.get("P21")
        first_name_prop = self.client.get("P735")
        name_prop = self.client.get("P734")
        gnd_uri_property = self.client.get("P227")
        try:
            self.first_name = str(self.entity[first_name_prop].label)
        except KeyError:
            self.first_name = None
        try:
            self.name = str(self.entity[name_prop].label)
        except KeyError:
            self.name = self.label
        try:
            self.date_of_birth = str(self.entity[date_of_birth_prop])
        except KeyError:
            self.date_of_birth = None
        try:
            self.date_of_death = str(self.entity[date_of_death_prop])
        except KeyError:
            self.date_of_death = None
        try:
            self.sex_or_gender = str(self.entity[sex_or_gender_prop].label)
        except KeyError:
            self.sex_or_gender = None
        try:
            place_of_birth_id = str(self.entity[place_of_birth_prop].id)
            self.place_of_birth = get_normalized_uri(
                f"{WIKIDATA_URL}{place_of_birth_id}"
            )
        except KeyError:
            self.place_of_birth = None
        try:
            place_of_death_id = str(self.entity[place_of_death_prop].id)
            self.place_of_death = get_normalized_uri(
                f"{WIKIDATA_URL}{place_of_death_id}"
            )
        except KeyError:
            self.place_of_death = None
        try:
            gnd_uri = self.entity[gnd_uri_property]
            self.gnd_uri = get_normalized_uri(f"{GND_URL}{gnd_uri}")
        except KeyError:
            self.gnd_uri = False



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()


requirements = [
    "acdh-arche-assets>=3.6,<4",
    "Wikidata>=0.7.0,<1",
]

setup_requirements = []

test_requirements = []

setup(
    author="Peter Andorfer",
    author_email="peter.andorfer@oeaw.ac.at",
    python_requires=">=3.10",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
    ],
    description="Python package to fetch data from Wikidata",
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    name="acdh-wikidata-pyutils",
    packages=find_packages(
        include=["acdh_wikidata_pyutils", "acdh_wikidata_pyutils.*"]
    ),
    setup_requires=setup_requirements,
    url="https://github.com/acdh-oeaw/acdh-wikidata-pyutils",
    version="v0.3.1",
    zip_safe=False,
)
